@@ -1,8 +1,8 @@
-/* $Id: nano.h,v 1.27.2.4 2002/01/10 13:47:04 astyanax Exp $ */
+/* $Id: nano.h,v 1.99 2003/02/13 22:25:01 astyanax Exp $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
- *   Copyright (C) 1999 Chris Allegretta                                  *
+ *   Copyright (C) 1999-2003 Chris Allegretta                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -19,10 +19,6 @@
  *                                                                        *
  **************************************************************************/
 
-#ifdef HAVE_MALLOC_H
-#include <malloc.h>
-#endif
-
 #ifdef HAVE_LIMITS_H
 #include <limits.h>
 #endif
@@ -34,21 +30,40 @@
 #define SET(bit) flags |= bit
 #define UNSET(bit) flags &= ~bit
 #define ISSET(bit) (flags & bit)
+#define TOGGLE(bit) flags ^= bit
 
+/* Define charalloc as a macro rather than duplicating code */
+#define charalloc(howmuch) (char *)nmalloc((howmuch) * sizeof(char))
+#define charealloc(ptr, howmuch) (char *)nrealloc(ptr, (howmuch) * sizeof(char))
+
+#ifndef NANO_SMALL
+  /* For the backup file copy ... */
+# define COPYFILEBLOCKSIZE 1024
+#endif
 
 #ifdef USE_SLANG	/* Slang support enabled */
 #include <slcurses.h>
-#define KEY_DC 0x113
+#define KEY_IC SL_KEY_IC
+#define KEY_DC SL_KEY_DELETE
 #elif defined(HAVE_NCURSES_H)
 #include <ncurses.h>
 #else /* Uh oh */
 #include <curses.h> 
 #endif /* CURSES_H */
 
-#ifdef HAVE_LIBINTL_H
-#include <libintl.h>
+#ifdef ENABLE_NLS
+#  ifdef HAVE_LIBINTL_H
+#    include <libintl.h>
+#  endif
+#  define _(string) gettext(string)
+#  define P_(singular, plural, number) ngettext(singular, plural, number)
+#else
+#  define _(string) (string)
+#  define P_(singular, plural, number) (number == 1 ? singular : plural)
 #endif
 
+#include <sys/types.h>
+#include <sys/stat.h>
 #include "config.h"
 
 #if !defined(HAVE_SNPRINTF) || !defined(HAVE_VSNPRINTF)
@@ -61,16 +76,73 @@
 # endif
 #endif
 
+#if !defined(HAVE_STRCASECMP) || !defined(HAVE_STRNCASECMP)
+# ifndef HAVE_STRCASECMP
+#  define strcasecmp strcmp
+# endif
+# ifndef HAVE_STRNCASECMP
+#  define strncasecmp strncmp
+# endif
+#endif
+
+/* HP-UX 10 & 11 do not seem to support KEY_HOME and KEY_END */
+#ifndef KEY_HOME
+#define KEY_HOME -1
+#endif /* KEY_HOME */
+
+#ifndef KEY_END
+#define KEY_END -1
+#endif /* KEY_END */
+
+/* Snatch these out of the ncurses defs, so we can use them in search
+   history regardless of whether we're using ncurses or not */
+#if !defined(KEY_UP) || !defined(KEY_DOWN)
+#define KEY_UP   0403
+#define KEY_DOWN 0402
+#endif /* !KEY_UP || !KEY_DOWN */
+
 #define VERMSG "GNU nano " VERSION
 
+#if defined(DISABLE_WRAPPING) && defined(DISABLE_JUSTIFY)
+#define DISABLE_WRAPJUSTIFY 1
+#endif
+
 /* Structure types */
 typedef struct filestruct {
     char *data;
     struct filestruct *next;	/* Next node */
     struct filestruct *prev;	/* Previous node */
-    long lineno;		/* The line number */
+    int lineno;			/* The line number */
 } filestruct;
 
+#ifdef ENABLE_MULTIBUFFER
+typedef struct openfilestruct {
+    char *filename;
+#ifndef NANO_SMALL
+    struct stat originalfilestat;
+#endif
+    struct openfilestruct *next;	/* Next node */
+    struct openfilestruct *prev;	/* Previous node */
+    struct filestruct *fileage;	/* Current file */
+    struct filestruct *filebot;	/* Current file's last line */
+#ifndef NANO_SMALL
+    struct filestruct *file_mark_beginbuf;
+				/* Current file's beginning marked line */
+    int file_mark_beginx;	/* Current file's beginning marked line's
+				   x-coordinate position */
+#endif
+    int file_current_x;		/* Current file's x-coordinate position */
+    int file_current_y;		/* Current file's y-coordinate position */
+    int file_flags;		/* Current file's flags: modification
+				   status (and marking status, if
+				   available) */
+    int file_placewewant;	/* Current file's place we want */
+    int file_totlines;		/* Current file's total number of lines */
+    long file_totsize;		/* Current file's total size */
+    int file_lineno;		/* Current file's line number */
+} openfilestruct;
+#endif
+
 typedef struct shortcut {
    int val;		/* Actual sequence that generates the keystroke */
    int altval;		/* Alt key # for this function, or 0 for none */
@@ -78,17 +150,72 @@ typedef struct shortcut {
    int misc2;
    int viewok;		/* is this function legal in view mode? */
    int (*func) (void);	/* Function to call when we catch this key */
-   char *desc;		/* Description, e.g. "Page Up" */
-   char *help;		/* Help file entry text */
+   const char *desc;	/* Description, e.g. "Page Up" */
+#ifndef DISABLE_HELP
+   const char *help;	/* Help file entry text */
+#endif
+   struct shortcut *next;
 } shortcut;
 
+#ifndef NANO_SMALL
 typedef struct toggle {
    int val;		/* Sequence to toggle the key.  Should only need 1 */
-   char *desc;		/* Description for when toggle is, uh, toggled,
+   const char *desc;	/* Description for when toggle is, uh, toggled,
 			   e.g. "Pico Messages"; we'll append Enabled or
 			   Disabled */
    int flag;		/* What flag actually gets toggled */
+   struct toggle *next;
 } toggle;
+#endif /* !NANO_SMALL */
+
+#ifdef ENABLE_NANORC
+typedef struct rcoption {
+   const char *name;
+   int flag;
+} rcoption;
+#endif /* ENABLE_NANORC */
+
+#ifdef ENABLE_COLOR
+
+typedef struct colortype {
+    int fg;			/* fg color */
+    int bg;			/* bg color */
+    int bright;			/* Is this color A_BOLD? */
+    int pairnum;		/* Color pair number used for this fg/bg */
+    regex_t start;		/* Start (or all) of the regex string */
+    regex_t *end;		/* End of the regex string */
+    struct colortype *next;
+} colortype;
+
+typedef struct exttype {
+    regex_t val;		/* The extensions that match this syntax. */
+    struct exttype *next;
+} exttype;
+
+typedef struct syntaxtype {
+    char *desc;			/* Name of this syntax type */
+    exttype *extensions;	/* List of extensions that this applies to */
+    colortype *color;		/* color struct for this syntax */
+    struct syntaxtype *next;
+} syntaxtype;
+
+#endif /* ENABLE_COLOR */
+
+#ifndef NANO_SMALL
+typedef struct historytype {
+    struct historytype *next;
+    struct historytype *prev;
+    char *data;
+} historytype;
+typedef struct historyheadtype {
+    struct historytype *next;	/* keep *next and *prev members together */
+    struct historytype *prev;	/* and in same order as in historytype */
+    struct historytype *tail;
+    struct historytype *current;
+    int count;
+    int len;
+} historyheadtype;
+#endif /* !NANO_SMALL */
 
 /* Bitwise flags so we can save space (or more correctly, not waste it) */
 
@@ -98,25 +225,35 @@ typedef struct toggle {
 #define MARK_ISSET		(1<<3)
 #define CONSTUPDATE		(1<<4)
 #define NO_HELP			(1<<5)
-#define PICO_MODE		(1<<6)
-#define FOLLOW_SYMLINKS		(1<<7)
-#define SUSPEND			(1<<8)
-#define NO_WRAP			(1<<9)
-#define AUTOINDENT		(1<<10)
-#define SAMELINEWRAP		(1<<11)
-#define VIEW_MODE		(1<<12)
-#define USE_MOUSE		(1<<13)
-#define USE_REGEXP              (1<<14)
-#define REGEXP_COMPILED         (1<<15)
-#define TEMP_OPT         	(1<<16)
-#define CUT_TO_END         	(1<<17)
-#define DISABLE_CURPOS         	(1<<18)
-/* 19, 20 in nano-1.1 */
-#define CLEAR_BACKUPSTRING    (1<<21)
-
+#define NOFOLLOW_SYMLINKS	(1<<6)
+#define SUSPEND			(1<<7)
+#define NO_WRAP			(1<<8)
+#define AUTOINDENT		(1<<9)
+#define SAMELINEWRAP		(1<<10)
+#define VIEW_MODE		(1<<11)
+#define USE_MOUSE		(1<<12)
+#define USE_REGEXP		(1<<13)
+#define REGEXP_COMPILED		(1<<14)
+#define TEMP_OPT		(1<<15)
+#define CUT_TO_END		(1<<16)
+#define REVERSE_SEARCH		(1<<17)
+#define MULTIBUFFER		(1<<18)
+#define DOS_FILE		(1<<19)
+#define MAC_FILE		(1<<20)
+#define SMOOTHSCROLL		(1<<21)
+#define DISABLE_CURPOS		(1<<22)	/* Damn, we still need it */
+#define ALT_KEYPAD		(1<<23)
+#define NO_CONVERT		(1<<24)
+#define BACKUP_FILE		(1<<25)
+#define NO_RCFILE		(1<<26)
+#define COLOR_SYNTAX		(1<<27)
+#define PRESERVE		(1<<28)
+#define HISTORY_CHANGED		(1<<29)
+#define HISTORYLOG		(1<<30)
 
 /* Control key sequences, changing these would be very very bad */
 
+#define NANO_CONTROL_SPACE 0
 #define NANO_CONTROL_A 1
 #define NANO_CONTROL_B 2
 #define NANO_CONTROL_C 3
@@ -175,6 +312,12 @@ typedef struct toggle {
 #define NANO_ALT_X 'x'
 #define NANO_ALT_Y 'y'
 #define NANO_ALT_Z 'z'
+#define NANO_ALT_PERIOD '.'
+#define NANO_ALT_COMMA ','
+#define NANO_ALT_LCARAT '<'
+#define NANO_ALT_RCARAT '>'
+#define NANO_ALT_BRACKET ']'
+#define NANO_ALT_SPACE ' '
 
 /* Some semi-changeable keybindings; don't play with unless you're sure you
 know what you're doing */
@@ -211,7 +354,6 @@ know what you're doing */
 #define NANO_FIRSTLINE_KEY	NANO_PREVPAGE_KEY
 #define NANO_LASTLINE_KEY	NANO_NEXTPAGE_KEY
 #define NANO_CANCEL_KEY		NANO_CONTROL_C
-#define NANO_CASE_KEY		NANO_CONTROL_A
 #define NANO_REFRESH_KEY	NANO_CONTROL_L
 #define NANO_JUSTIFY_KEY	NANO_CONTROL_J
 #define NANO_JUSTIFY_FKEY	KEY_F(4)
@@ -231,53 +373,59 @@ know what you're doing */
 #define NANO_ENTER_KEY		NANO_CONTROL_M
 #define NANO_FROMSEARCHTOGOTO_KEY NANO_CONTROL_T
 #define NANO_TOFILES_KEY	NANO_CONTROL_T
+#define NANO_APPEND_KEY		NANO_ALT_A
+#define NANO_PREPEND_KEY	NANO_ALT_P
+#define NANO_OPENPREV_KEY	NANO_ALT_LCARAT
+#define NANO_OPENNEXT_KEY	NANO_ALT_RCARAT
+#define NANO_OPENPREV_ALTKEY	NANO_ALT_COMMA
+#define NANO_OPENNEXT_ALTKEY	NANO_ALT_PERIOD
+#define NANO_BRACKET_KEY	NANO_ALT_BRACKET
+#define NANO_EXTCMD_KEY		NANO_CONTROL_X
+#define NANO_NEXTWORD_KEY	NANO_CONTROL_SPACE
+#define NANO_PREVWORD_KEY	NANO_ALT_SPACE
 
+#ifndef NANO_SMALL
+/* Toggles do not exist with NANO_SMALL. */
 #define TOGGLE_CONST_KEY	NANO_ALT_C
 #define TOGGLE_AUTOINDENT_KEY	NANO_ALT_I
 #define TOGGLE_SUSPEND_KEY	NANO_ALT_Z
 #define TOGGLE_NOHELP_KEY	NANO_ALT_X
-#define TOGGLE_PICOMODE_KEY	NANO_ALT_P
 #define TOGGLE_MOUSE_KEY	NANO_ALT_M
 #define TOGGLE_CUTTOEND_KEY	NANO_ALT_K
-#define TOGGLE_REGEXP_KEY	NANO_ALT_E
+#define TOGGLE_REGEXP_KEY	NANO_ALT_R
 #define TOGGLE_WRAP_KEY		NANO_ALT_W
+#define TOGGLE_BACKWARDS_KEY	NANO_ALT_B
+#define TOGGLE_CASE_KEY		NANO_ALT_C
+#define TOGGLE_LOAD_KEY		NANO_ALT_F
+#define TOGGLE_DOS_KEY		NANO_ALT_D
+#define TOGGLE_MAC_KEY		NANO_ALT_O
+#define TOGGLE_SMOOTH_KEY	NANO_ALT_S
+#define TOGGLE_NOCONVERT_KEY	NANO_ALT_N
+#define TOGGLE_BACKUP_KEY	NANO_ALT_B
+#define TOGGLE_SYNTAX_KEY	NANO_ALT_Y
+#endif /* !NANO_SMALL */
 
-#define MAIN_LIST_LEN 26
 #define MAIN_VISIBLE 12
-#define WHEREIS_LIST_LEN 6
-#define REPLACE_LIST_LEN 6
-#define REPLACE_LIST_2_LEN 3
-#define GOTO_LIST_LEN 3
-#define HELP_LIST_LEN 3
-#define SPELL_LIST_LEN 1
-
-#ifndef DISABLE_BROWSER
-#define WRITEFILE_LIST_LEN 2
-#define BROWSER_LIST_LEN 3
-#else
-#define WRITEFILE_LIST_LEN 1
-#endif
-
-#ifdef HAVE_REGEX_H
-#define TOGGLE_LEN 9
-#else
-#define TOGGLE_LEN 8
-#endif
 
 #define VIEW 1
 #define NOVIEW 0
 
-#define TOP 2
-#define CENTER 1
-#define BOTTOM 0
+typedef enum {
+    CENTER, TOP, NONE
+} topmidbotnone;
 
-/* Minimum editor window rows required for Nano to work correctly */
+/* Minimum editor window rows required for nano to work correctly */
 #define MIN_EDITOR_ROWS 3
 
-/* Default number of characters from end-of-line where text wrapping occurs */
+/* Minimum editor window cols required for nano to work correctly */
+#define MIN_EDITOR_COLS 10
+
+/* Default number of characters from end-of-line where text wrapping
+   occurs */
 #define CHARS_FROM_EOL 8
 
-/* Minimum fill length (space available for text before wrapping occurs) */
-#define MIN_FILL_LENGTH 10
+/* Maximum number of search history strings saved, same value used for
+   replace history */
+#define MAX_SEARCH_HISTORY 100
 
-#endif /* ifndef NANO_H */ 
+#endif /* !NANO_H */
